﻿using PlcSiemens.Core.Extension;
using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using SqlSugar;
using System.ComponentModel;
using WCS.Core;
using WCS.Entity;
using WCS.Entity.Protocol.Station;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;
using TaskStatus = WCS.Entity.TaskStatus;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  无交互系统
    /// </summary>
    [BelongTo(typeof(NoInteractionWorld))]
    [Description("无交互系统")]
    public class NoInteractionSystems : DeviceSystem<Device<IStation520>>
    {
        protected override bool ParallelDo => true;

        public override void Do(Device<IStation520> obj)
        {
            var taskInfos = new List<int>();
            var sqlSugar = new SqlSugarHelper();

            SqlSugarHelper.Do(db =>
            {
                //获取所有的新建任务，组盘任务不需要
                taskInfos = db.Default.Queryable<WCS_TaskInfo>().ReadPastUpdLock().Where(x => x.Status == 0).ToList().Where(x => x.Type != TaskType.SetPlate || (x.Type == TaskType.SetPlate && x.AddrFrom != "Robot")).Select(x => x.ID).ToList();
                var time = DateTime.Now.AddMinutes(-20);
                var timeOut = db.Default.Queryable<WCS_TaskInfo>().NoLock().Count(x => x.BusType == "车间叫料" && x.Status < TaskStatus.StackerExecution && x.EditTime < time);
                if (timeOut > 0) World.Log($"共有{timeOut}个车间叫料任务超过二十分钟未执行，请检查相关相关任务对应的堆垛机是否正常、堆垛机放货点是否卡任务等", LogLevelEnum.High);
            });
            if (!taskInfos.Any())
            {
                return;
            }

            var isEnd = false; //每个周期只处理一个任务
            foreach (var item in taskInfos)
            {
                if (isEnd) return;

                try
                {
                    SqlSugarHelper.Do(db =>
                    {
                        var task = db.Default.Queryable<WCS_TaskInfo>().Where(t => t.ID == item).First();
                        switch (task.Type)
                        {
                            case TaskType.SetPlate:
                                if (task.AddrFrom != "Robot")
                                {
                                    //更新任务状态
                                    task.Status = TaskStatus.WaitingToExecute;
                                    db.Default.UpdateableRowLock(task).UpdateColumns(x => new { x.Status }).ExecuteCommand();
                                    task.AddWCS_TASK_DTL(db.Default, task.Device, $"初始化入库任务信息");
                                    isEnd = true;
                                }
                                break;

                            case TaskType.EnterDepot:
                                //更新任务状态
                                task.Status = Entity.TaskStatus.WaitingToExecute;
                                task.Device = task.WarehouseCode switch
                                {
                                    "1N" => "SRM1",
                                    "1S" => "SRM2",
                                    "2N" => "SRM3",
                                    "2S" => "SRM4",
                                    "3N" => "SRM5",
                                    "3S" => "SRM6",
                                    _ => task.Device
                                };

                                task.Device = task.AddrFrom switch
                                {
                                    "1666" or "1661" => "SRM1",
                                    "1681" or "1676" => "SRM2",
                                    "1696" or "1691" => "SRM3",
                                    "1711" or "1706" => "SRM4",
                                    "1726" or "1721" => "SRM5",
                                    "1741" or "1736" => "SRM6",
                                    _ => task.Device
                                };

                                //计算下一个地址
                                var path1 = DevicePath.GetPath(task.AddrFrom, task.Device);
                                task.AddrNext = path1.Points[1].Code;
                                task.SrmStation = task.BarCode.Contains("TPA") || task.BarCode.Contains("TPB") ? task.AddrFrom : path1.Points[2].Code;

                                db.Default.UpdateableRowLock(task).UpdateColumns(x => new { x.Status, x.Device, x.AddrNext, x.SrmStation }).ExecuteCommand();
                                task.AddWCS_TASK_DTL(db.Default, task.Device, $"初始化入库任务信息");
                                isEnd = true;
                                break;

                            case TaskType.OutDepot:
                                {
                                    if (task.Device.Contains("Robot"))
                                    {
                                        var lastTask = db.Default.Queryable<WCS_TaskInfo>().NoLock().First(x => x.Type == TaskType.OutDepot && x.AddrTo == task.AddrTo && x.Status > TaskStatus.WaitingToExecute && x.Status < TaskStatus.Finish);
                                        if (lastTask != null)
                                        {
                                            World.Log($"等待任务结束:{task.AddrTo}上一组码垛任务未结束，等待{lastTask.ID}任务完成后开始初始化当前任务{task.ID}");
                                            return;
                                        }
                                        var pos = task.AddrFrom.Split("-");
                                        task.Status = Entity.TaskStatus.WaitingToExecute;
                                        task.Line = pos[0].ToShort();
                                        task.Col = pos[1].ToShort();
                                        task.Layer = pos[2].ToShort();
                                        task.Depth = pos[3].ToShort();
                                        db.Default.UpdateableRowLock(task).UpdateColumns(x => new { x.Status, x.Line, x.Col, x.Layer, x.Depth }).ExecuteCommand();
                                        task.AddWCS_TASK_DTL(db.Default, task.Device, $"初始化出库任务信息,放货站台:{task.SrmStation}");
                                        isEnd = true;
                                    }
                                    else
                                    {
                                        if (task.SrmStation == "1")
                                        {
                                            var srmStation = new string[2];
                                            switch (task.Device)
                                            {
                                                case "SRM1":
                                                    srmStation[0] = "2534";
                                                    srmStation[1] = "2533";
                                                    break;

                                                case "SRM2":
                                                    srmStation[0] = "2734";
                                                    srmStation[1] = "2733";
                                                    break;

                                                case "SRM3":
                                                    srmStation[0] = "2934";
                                                    srmStation[1] = "2933";
                                                    break;

                                                case "SRM4":
                                                    srmStation[0] = "3134";
                                                    srmStation[1] = "3133";

                                                    break;

                                                case "SRM5":
                                                    srmStation[0] = "3334";
                                                    srmStation[1] = "3333";
                                                    break;

                                                case "SRM6":
                                                    srmStation[0] = "3534";
                                                    srmStation[1] = "3533";
                                                    break;
                                            }

                                            //开始计算当前这个任务要从哪个站台出
                                            //计算两个站台小于取货完成数量的AGV任务
                                            var agv1 = db.Default.Queryable<WCS_TaskInfo>().NoLock().Where(x => x.Status <= TaskStatus.StackerExecution && x.AddrTo == srmStation[0]).Count();
                                            var agv2 = db.Default.Queryable<WCS_TaskInfo>().NoLock().Where(x => x.Status <= TaskStatus.StackerExecution && x.AddrTo == srmStation[1]).Count();
                                            task.SrmStation = "";
                                            task.AddrTo = agv1 > agv2 ? srmStation[1] : srmStation[0];
                                        }
                                        if (task.SrmStation.IsNullOrEmpty()) //如果没有指定放货站台
                                        {
                                            if (task.Device.IsNullOrEmpty())
                                            {
                                                task.Device = "SRM" + task.Tunnel.GetLastDigit();
                                            }
                                            //获取堆垛机到目标地址的路径信息
                                            var path = DevicePath.GetPath(task.Device, task.AddrTo);
                                            task.SrmStation = path.Points[1].Code;
                                        }

                                        var devs = Device.All.Where(x => x.HasFlag(DeviceFlags.AGV取货站台口)).Select(x => x.Code);
                                        if (devs.Contains(task.SrmStation))
                                        {
                                            var b = new WCS_AgvTaskInfo()
                                            {
                                                ID = db.GetAgvTaskId(),
                                                TaskType = AGVTaskType.EnterDepot,
                                                Status = AGVTaskStatus.NewBuild,
                                                TaskId = task.ID,
                                                Position = task.WorkBench,
                                                Station = task.WarehouseCode switch
                                                {
                                                    "1N" => "2501",
                                                    "1S" => "2701",
                                                    "2N" => "2901",
                                                    "2S" => "3101",
                                                    "3N" => "3301",
                                                    "3S" => "3501",
                                                    _ => throw new NotImplementedException(),
                                                },
                                                AddWho = "WCS",
                                                AddTime = DateTime.Now
                                            };
                                            db.Default.InsertableRowLock(b).SplitTable().ExecuteCommand();
                                        }
                                        //更新任务状态
                                        task.Status = Entity.TaskStatus.WaitingToExecute;
                                        db.Default.UpdateableRowLock(task).UpdateColumns(x => new { x.SrmStation, x.AddrTo, x.Status }).ExecuteCommand();
                                        task.AddWCS_TASK_DTL(db.Default, task.Device, $"初始化出库任务信息,放货站台:{task.SrmStation}");
                                        isEnd = true;
                                    }
                                    break;
                                }
                            case TaskType.Delivery:
                                break;

                            case TaskType.EmptyInit:
                                break;
                        }
                    });
                }
                catch (Exception ex)
                {
                    World.Log(ex.Message, LogLevelEnum.Mid);
                }
            }
        }

        public override bool Select(Device dev)
        {
            return dev.Code == nameof(NoInteractionSystems);
        }
    }
}
